@@ -1,8 +1,10 @@
 #!/usr/local/bin/perl -w
 
+use ExtUtils::testlib;
+
 die "Use 'make test' to run test.pl\n" unless "@INC" =~ /\bblib\b/;
 
-# $Id: test.pl,v 1.29 1997/01/14 21:48:19 timbo Exp $
+# $Id: test.pl,v 1.30 1997/06/14 17:42:12 timbo Exp $
 #
 # Copyright (c) 1995, Tim Bunce
 #
@@ -12,10 +14,11 @@ die "Use 'make test' to run test.pl\n" unless "@INC" =~ /\bblib\b/;
 require 'getopts.pl';
 
 $| = 1;
-print q{Oraperl test application $Revision: 1.29 $}."\n";
+print q{Oraperl test application $Revision: 1.30 $}."\n";
 
 $SIG{__WARN__} = sub {
-	($_[0] =~ /^Bad free/) ? warn "See README about Bad free() warnings!\n": warn @_;
+	($_[0] =~ /^(Bad|Duplicate) free/)
+		? warn "\n*** Read the README file about Bad free() warnings!\n": warn @_;
 };
 
 $opt_d = 0;		# debug
@@ -23,8 +26,7 @@ $opt_l = 0;		# log
 $opt_n = 5;		# num of loops
 $opt_m = 0;		# count for mem leek test
 $opt_c = 1;		# do cache test
-$opt_p = 1;		# test pl/sql code
-&Getopts('m:d:n:clp ') || die "Invalid options\n";
+&Getopts('m:d:n:cl ') || die "Invalid options\n";
 
 $ENV{PERL_DBI_DEBUG} = 2 if $opt_d;
 $ENV{ORACLE_HOME} = '/usr/oracle' unless $ENV{ORACLE_HOME};
@@ -68,7 +70,7 @@ printf("(ORACLE_SID='%s', TWO_TASK='%s')\n", $ENV{ORACLE_SID}||'', $ENV{TWO_TASK
 $start = time;
 
 rename("test.log","test.olog") if $opt_l;
-eval 'DBI->internal->{DebugLog} = "test.log";'  if $opt_l;
+eval 'DBI->_debug_dispatch(3,"test.log");' if $opt_l;
 
 &test3($opt_m) if $opt_m;
 
@@ -92,8 +94,6 @@ print "done.\n";
 
 &test_cache() if $opt_c;
 
-&test_plsql() if $opt_p;
-
 $dur = time - $start;
 print "\nTest complete ($dur seconds).\n";
 print "If the tests above have produced the 'expected' output then they have passed.\n";
@@ -241,85 +241,4 @@ sub count_fetch {
 }
 
 
-sub test_plsql {
-    print "\nTesting PL/SQL interaction.\n";
-    local($l) = &ora_login($dbname, $dbuser, '')
-		    || die "ora_login: $ora_errno: $ora_errstr\n";
-    my $c;
-
-	# $l->debug(2);
-    $c = &ora_open($l, q{
-	    begin RAISE invalid_number; end;
-    });
-    # Expect ORA-01722: invalid number
-    die "ora_open: $ora_errstr" unless $ora_errno == 1722;
-
-    $c = &ora_open($l, q{
-	    DECLARE FOO EXCEPTION;
-	    begin raise foo; end;
-    });
-    # Expect ORA-06510: PL/SQL: unhandled user-defined exception
-    die "ora_open: $ora_errstr" unless $ora_errno == 6510;
-
-    $c = &ora_open($l, q{
-	    begin raise_application_error(-20101,'app error'); end;
-    });
-    # Expect our exception number and error text
-    die "ora_open: $ora_errno $ora_errstr"
-	    unless $ora_errno == 20101;			# our exception number
-    die "ora_open: $ora_errstr"
-	    unless $ora_errstr =~ m/app error/;	# our exception text
-
-    $c = &ora_open($l, q{
-	declare err_num number; err_msg char(510);
-	begin
-	    err_num := :1;
-	    err_msg := :2;
-	    raise_application_error(-20000-err_num, 'plus '||err_msg);
-	end;
-    }) || die "ora_open: $ora_errstr";
-    $c->execute(42,"my msg");
-    # Expect our exception number and error text
-    die "ora_open: $ora_errno $ora_errstr"
-	    unless $ora_errno == 20042;				# our exception number
-    die "ora_open: $ora_errstr"
-	    unless $ora_errstr =~ m/plus my msg/;	# our exception text
-
-    print "Testing numeric bind_param_inout. Expect '200', '3800', '75800':\n";
-    #$l->debug(2);
-    #DBI->internal->{DebugDispatch} = 2;
-    $c = &ora_open($l, q{
-	declare bar number;
-	begin bar := :1; bar := bar * 20; :1 := bar; end;
-    }) || die "ora_open: $ora_errstr";
-    $param = 10;
-    $c->bind_param_inout(1, \$param, 100) || die "bind_param_inout $ora_errstr";
-    do {
-	$c->execute	|| die "execute $ora_errstr";
-	print "param='$param'\n";
-	$param -= 10;
-    } while ($param < 70000);
-
-    print "Testing string bind_param_inout. Expect '**foo**':\n";
-    #$l->debug(2);
-    #DBI->internal->{DebugDispatch} = 2;
-    $c = &ora_open($l, q{
-	declare bar varchar2(200);
-	begin bar := :1; bar := '**'||bar||'**'; :1 := bar; end;
-    }) || die "ora_open: $ora_errstr";
-    $param = "foo";
-    $c->bind_param_inout(1, \$param, 100) || die "bind_param_inout $ora_errstr";
-	# can be used for force realloc and thus a 'mutated location' error:
-    # $param = "ffffffffffffffffffffffffffffffffffffffffffffffffoo" x 3;
-    $c->execute	|| die "execute $ora_errstr";
-    print "param='$param'\n";
-
-    # To do
-    #	test NULLs at first bind
-    #	NULLS later binds.
-    #	returning NULLS
-    #	multiple params, mixed types and in only vs inout
-    #	automatic rebind if location changes
-}
-
 # end.
